/*
 * Test program for syntax.c: reads a file on stdin or from argv[1] and
 * uses HTML to print it in pretty colors.
 *
 * Copyright (C) 1998,1999,2004 by Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "syntax.h"
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* order matters here.  See the enum in syntax.h */
const char *highlight[] = {
  "<font color=\"#ffff00\">",  /* yellow */
  "<font color=\"#ffffff\"><b>",  /* white */
  "<font color=\"#c0c0c0\"><i>",  /* grey */
  "<font color=\"#00ffff\">",  /* b-cyan */
  "<font color=\"#00ffff\">",  /* b-cyan */
  "<font color=\"#00ff00\">",  /* b-green */
  "<font color=\"#b09000\">",  /* brown */
  "<font color=\"#00c0c0\">",  /* plain cyan */
  "<font color=\"#800000\">",  /* b-red */
  "<font color=\"#808080\">",  /* whitespace */
};
const char *unlight[] = {
  "</font>",
  "</b></font>",
  "</i></font>",
  "</font>",
  "</font>",
  "</font>",
  "</font>",
  "</font>",
  "</font>",
  "</font>"
};

int main(int argc, char **argv) {
  int i, di, line=1;
  char *space = "\xb7", *tab = "\xbb  ";
  syntax_t cur = NONE;
  FILE *fin;
  char *filename = "&lt;untitled&gt;";
  char buf[4096];  /* there had better not be any lines longer than this! */

  language = CPP;

  if (argc > 2) {
    printf("Usage:\n  syntax foo.c\n\n");
    return -1;
  }

  if (argc == 2) {
    char *p;
    if (!(fin = fopen(argv[1], "r"))) {
      perror("fopen");
      return -1;
    }
    filename = argv[1];
    if ((p = strrchr(filename, '/')) != NULL)
      filename = p+1;
  }
  else fin=stdin;

  printf("<html>\n<head>\n<title> %s </title>\n", filename);
  printf("<body bgcolor=\"#000030\" text=\"#c0c0c0\">\n");
  printf("<!-- generated by chtml, from Patrick Reynolds' C/C++ highlighting kit -->\n");
  printf("<!-- http://www.cs.duke.edu/~reynolds/projects/#syntax -->\n");
  printf("<pre>\n");
  while (fgets(buf, 4096, fin)) {
    if (cur != NONE) { printf(unlight[cur]); cur = NONE; }
    printf("%5d: ", line++); fflush(stdout);
    for (i=0; buf[i]; ) {
      if (buf[i] != '\n' && buf[i] != '\r') {
        syntax_t syn = get_syntax(buf, i, &di, !i);
        if (syn != cur) {
          if (cur != NONE) printf(unlight[cur]);
          printf(highlight[syn]);
          cur = syn;
        }
      }
      else {
        di = 1;
        if (cur != NONE) {
          printf(unlight[cur]);
          cur = NONE;
        }
      }
      if (cur == WHITESPACE) {
        int j;
        for (j=0; j<di; j++)
          if (buf[i+j] == ' ')
            printf(space);
          else if (buf[i] == '\t')
            printf(tab);
      }
      else {
        int j;
        for (j=0; j<di; j++)
          if (buf[i+j] == '>') printf("&gt;");
          else if (buf[i+j] == '<') printf("&lt;");
          else if (buf[i+j] == '&') printf("&amp;");
          else putchar(buf[i+j]);
      }
      i += di;
    }
  }

  if (cur != NONE) printf("%s\n", unlight[cur]);
  printf("</pre>\n</body>\n</html>\n");

  return 0;
}
